@@ -21,64 +21,76 @@ from copy import deepcopy
 import pyvista as pv
 import random
 from tqdm import tqdm
+from time import perf_counter
 
 def compete_add(forest,network_ids=-1,radius_buffer=0.05,exact=True):
     #Find a viable new point to add within
-    all_network_ids = list(range(forest.number_of_networks))
     if network_ids == -1:
-        network_ids = all_network_ids
+        active_network_ids = list(range(forest.number_of_networks))
+    else:
+        active_network_ids = [network_ids]
     #points,_ = forest.boundary.pick(size=prefetch,homogeneous=True,replacement=False)
     #points = points.tolist()
-    active_network_ids = deepcopy(network_ids)
-    active_network_tree_ids = [list(range(forest.trees_per_network[net])) for net in all_network_ids]
-    scale = len(network_ids)
+    #active_network_ids = deepcopy(network_ids)
+    active_network_tree_ids = [list(range(forest.trees_per_network[net])) for net in active_network_ids]
+    #scale = len(network_ids)
     volume = deepcopy(forest.boundary.volume)
-    forest.boundary.volume = forest.boundary.volume/scale
+    #forest.boundary.volume = forest.boundary.volume/scale
     #networks_copy = deepcopy(forest.networks)
-    unused_networks = list(set(all_network_ids) - set(active_network_ids))
+    #unused_networks = list(set(all_network_ids) - set(active_network_ids))
     while len(active_network_ids) > 0:
         for nid in active_network_ids:
             for njd in active_network_tree_ids[nid]:
-                ## New
-                if  forest.networks[nid][njd].data.shape[0] < 100:
-                    if len(forest.networks[nid][njd].rng_points) <= 1:
-                        forest.networks[nid][njd].rng_points,_ = forest.boundary.pick(size=min(len(forest.boundary.tet_verts),10000),homogeneous=True,replacement=False)
-                        forest.networks[nid][njd].rng_points = forest.networks[nid][njd].rng_points.tolist()
-                    for n in range(len(forest.networks[nid][njd].rng_points)):
-                        pt = np.array(forest.networks[nid][njd].rng_points.pop(0))
-                        if exact:
-                            other_vessels,distances = close_exact(forest.networks[nid][njd].data,pt)
-                        else:
-                            other_vessels,distances = close(forest.networks[nid][njd].data,pt)
-                        minimum_distance = min(distances)
-                        retry = False
-                        for idx in network_ids:
+                ## Preallocate points to check for new vessel appending
+                while len(forest.networks[nid][njd].rng_points) <= 1:
+                    for idx in list(range(forest.number_of_networks)):
+                        for jdx in list(range(forest.trees_per_network[idx])):
+                            forest.networks[idx][jdx].rng_points = []
+                    rng_points,_ = forest.boundary.pick(size=min(len(forest.boundary.tet_verts),10000),homogeneous=True,replacement=False)
+                    rng_points = rng_points.tolist()
+                    for n in range(len(rng_points)):
+                        pt = np.array(rng_points.pop(0))
+                        closest_network  = None
+                        closest_distance = np.inf
+                        for idx in list(range(forest.number_of_networks)):
                             for jdx in list(range(forest.trees_per_network[idx])):
-                                if idx == nid:
-                                    continue
-                                if forest.networks[nid][njd].data.shape[0] < 100:
-                                    other_vessels,distances = close_exact(forest.networks[idx][jdx].data,pt)
+                                if exact:
+                                    distances = close_exact_v2(forest.networks[idx][jdx].data,pt)
                                 else:
-                                    other_vessels,distances = close(forest.networks[idx][jdx].data,pt)
-                                if min(distances) < minimum_distance:
-                                    retry = True
-                                    break
-                            if retry:
-                                break
-                        if retry:
-                            continue
-                        else:
-                            forest.networks[nid][njd].rng_points.insert(-1,pt.tolist())
+                                    _,distances = close(forest.networks[idx][jdx].data,pt)
+                                if min(distances) < closest_distance:
+                                    closest_network = idx
+                                    closest_distance = min(distances)
+                        forest.networks[closest_network][np.random.choice(len(forest.networks[closest_network]))].rng_points.insert(-1,pt.tolist())
+                start = perf_counter()
+                for n in range(len(forest.networks[nid][njd].rng_points)):
+                    pt = np.array(forest.networks[nid][njd].rng_points.pop(0))
+                    closest_network  = None
+                    closest_distance = np.inf
+                    for idx in list(range(forest.number_of_networks)):
+                        for jdx in list(range(forest.trees_per_network[idx])):
+                            if exact:
+                                distances = close_exact_v2(forest.networks[idx][jdx].data,pt)
+                            else:
+                                _,distances = close(forest.networks[idx][jdx].data,pt)
+                            if min(distances) < closest_distance:
+                                closest_network = idx
+                                closest_distance = min(distances)
+                    forest.networks[closest_network][np.random.choice(len(forest.networks[closest_network]))].rng_points.insert(-1,pt.tolist())
+                print('End point reallocation: {}'.format(perf_counter()-start))
                 ##
                 #print(len(forest.networks[nid][njd].rng_points))
                 #print("Check: {}".format(forest.networks[nid][njd].rng_points[0]))
+                start = perf_counter()
                 vessel,data,sub_division_map,sub_division_index,threshold = forest.networks[nid][njd].add(-1,0,isforest=True,radius_buffer=radius_buffer)
+                print('End vessel appending: {}'.format(perf_counter()-start))
                 point = data[-2,3:6] #new terminal point
                 #print(point)
-                closest_network = nid
-                _,closest_value = close_exact(forest.networks[nid][njd].data[vessel,:].reshape(1,forest.networks[nid][njd].data.shape[1]),point)
-                escape = False
-                for idx in network_ids:
+                #closest_network = nid
+                #_,closest_value = close_exact(forest.networks[nid][njd].data[vessel,:].reshape(1,forest.networks[nid][njd].data.shape[1]),point)
+                #escape = False
+                """
+                for idx in list(range(forest.number_of_networks)):
                     for jdx in list(range(forest.trees_per_network[idx])):
                         if idx == nid:
                             continue
@@ -89,6 +101,7 @@ def compete_add(forest,network_ids=-1,radius_buffer=0.05,exact=True):
                         minimum_distance = min(distances)
                         minimum_vessel_point = (forest.networks[idx][jdx].data[other_vessels[0],0:3] + forest.networks[idx][jdx].data[other_vessels[0],3:6])/2
                         if minimum_distance < closest_value:
+                            #print('other closer')
                             vessel_within = True
                             if not forest.convex:
                                 subdivisions = 5
@@ -96,6 +109,7 @@ def compete_add(forest,network_ids=-1,radius_buffer=0.05,exact=True):
                                     tmp = point*(sub/(2*subdivisions)) + minimum_vessel_point*(1-sub/(2*subdivisions))
                                     if not forest.boundary.within(tmp[0],tmp[1],tmp[2],2):
                                         vessel_within = False
+                                        #print('not within')
                                         break
                             if not vessel_within:
                                 continue
@@ -104,13 +118,16 @@ def compete_add(forest,network_ids=-1,radius_buffer=0.05,exact=True):
                                 break
                     if escape:
                         break
-                if escape:
-                    #forest.networks[nid][njd] = deepcopy(networks_copy[nid][njd])
-                    continue
+                """
+                #if escape:
+                #    #forest.networks[nid][njd] = deepcopy(networks_copy[nid][njd])
+                #    continue
+                start = perf_counter()
                 new_vessels = np.vstack((data[vessel,:],data[-2,:],data[-1,:]))
                 repeat = False
-                check_networks = unused_networks + [nid]
-                for nikd in range(len(unused_networks)):
+                #check_networks = unused_networks + [nid]
+                check_networks = [nid]
+                for nikd in check_networks:
                     if nikd == nid:
                         check_trees = list(range(len(forest.networks[nikd])))
                         check_trees.remove(njd)
@@ -122,6 +139,7 @@ def compete_add(forest,network_ids=-1,radius_buffer=0.05,exact=True):
                             break
                     if repeat:
                         break
+                print('End collision testing: {}'.format(perf_counter()-start))
                 if repeat:
                     #forest.networks[nid][njd] = deepcopy(networks_copy[nid][njd])
                     continue
